@@ -60,7 +60,7 @@ setValidity2("RangedSummarizedExperiment", .valid.RangedSummarizedExperiment)
 .new_RangedSummarizedExperiment <- function(assays, rowRanges, colData,
                                             metadata)
 {
-    assays <- Assays(assays, as.null.if.zero.assay=TRUE)
+    assays <- Assays(assays, as.null.if.no.assay=TRUE)
     elementMetadata <- S4Vectors:::make_zero_col_DataFrame(length(rowRanges))
     new("RangedSummarizedExperiment", rowRanges=rowRanges,
                                       colData=colData,
@@ -86,9 +86,13 @@ setValidity2("RangedSummarizedExperiment", .valid.RangedSummarizedExperiment)
 SummarizedExperiment <- function(assays=SimpleList(),
                                  rowData=NULL, rowRanges=GRangesList(),
                                  colData=DataFrame(),
-                                 metadata=list())
+                                 metadata=list(),
+                                 checkDimnames=TRUE)
 {
-    assays <- normarg_assays(assays, as.null.if.zero.assay=TRUE)
+    if (!isTRUEorFALSE(checkDimnames))
+        stop(wmsg("'checkDimnames' must be TRUE or FALSE"))
+
+    assays <- normarg_assays(assays, as.null.if.no.assay=TRUE)
 
     if (!missing(colData)) {
         if (!is(colData, "DataFrame"))
@@ -127,11 +131,29 @@ SummarizedExperiment <- function(assays=SimpleList(),
     }
 
     if (missing(rowRanges) && !is(rowData, "GenomicRanges_OR_GRangesList")) {
-        new_SummarizedExperiment(assays, ans_rownames, rowData, colData,
-                                 metadata)
+        ans <- new_SummarizedExperiment(assays, ans_rownames, rowData, colData,
+                                        metadata)
     } else {
-        .new_RangedSummarizedExperiment(assays, rowRanges, colData, metadata)
+        ans <- .new_RangedSummarizedExperiment(assays, rowRanges, colData,
+                                               metadata)
     }
+    if (!checkDimnames)
+        return(ans)
+    ans_dimnames <- dimnames(ans)
+    ok <- assays_have_expected_dimnames(ans@assays, ans_dimnames, strict=FALSE)
+    if (!ok) {
+        if (is.null(ans_dimnames[[1L]])) {
+            what <- "colnames"
+        } else if (is.null(ans_dimnames[[2L]])) {
+            what <- "rownames"
+        } else {
+            what <- "rownames and colnames"
+        }
+        stop(wmsg("the ", what, " of the supplied assay(s) must be NULL ",
+                  "or identical to those of the ", class(ans), " object ",
+                  "(or derivative) to construct"))
+    }
+    ans
 }
 
 
@@ -234,13 +256,6 @@ setReplaceMethod("names", "RangedSummarizedExperiment",
     BiocGenerics:::replaceSlots(x, rowRanges=rowRanges, check=FALSE)
 })
 
-setMethod("dimnames", "RangedSummarizedExperiment",
-    function(x)
-{
-    ans <- list(names(x), rownames(colData(x)))
-    DelayedArray:::simplify_NULL_dimnames(ans)
-})
-
 setReplaceMethod("dimnames", c("RangedSummarizedExperiment", "list"),
     function(x, value)
 {
